#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on 6/26/18 

@author: Sanjana Kapoor
@author: John Sigmon

Last modified: 6/28/18
"""

import os
import sys
import json
import pickle as pkl
import spacy
import logging.config
from spacy.attrs import ORTH, LEMMA
from spacy.tokens import Doc

logger = logging.getLogger()
ROOT_PATH = os.path.dirname(os.path.realpath(__file__))

class TokenFilter():
    
    apply = None
    dispatch_fun = None

    # Default params
    to_lower = True

    # Supported params
    keep_alpha = False          # Keep only alphas
    keep_alpha_nums = True      # Keep only alphas and digits
    remove_stops = True
    remove_oov = False           # Out of Vocab
    remove_nums = False          # Removes anything resembling number
    add_special_case = None
    remove_url = True
    remove_email = True
    remove_punct = False
    threshold = None # Exclusive threshold

    def __init__(self, name, **params):
        '''

        :param name {str} name of the filtering you wish to use

        :param **params supported keywords are above as attributes

        '''
        
        self.dispatch_fun = {
                'spacy' : self.spacy,
                'nltk' : self.nltk,
                'frequency': self.frequency
                }
        
        # Choose library
        if name: 
            self.apply = self.dispatch_fun[name]
        
        # Parse params
        for key in params:
            setattr(self, key, params[key])
        logger.debug(vars(self))
        
        # Setup big files for vocab etc
        if name == 'spacy':
            #import en_core_web_md
            #self.nlp = en_core_web_md.load()
            import en_core_web_sm
            self.nlp = en_core_web_sm.load()
        
        if name == 'nltk':
            from nltk.corpus import words
            from nltk.corpus import stopwords
            if self.remove_stops is True:
                self.stop_words = set(stopwords.words('english'))

            if self.remove_oov is True:
                self.vocab = words.words() 
        
    def spacy(self, docs):
        '''
        Spacy tokenizer supports adding special cases. 

        The method takes the input docs, tokenizes them, and returns a list
        of lists of strings (tokens).

        : params {listlist[str]} documents to be filtered

        : returns {list[list[str]]} tokenized documents

        '''
        

        if self.add_special_case is not None:
            special_cases = self.add_special_case
            for pattern, replace in special_cases:
                logger.debug(type(pattern, replace))
                self.nlp.tokenizer.add_special_case(pattern, replace)
        
        # Tokenize
        tkns = []
        for doc in docs:
            # Intermediate steps are all tokens. Lemmatize at the end.
            # Add unicode text to tkns at end of intermediate steps
            # Pass over each doc multiple times to avoid complicated logic.
            doc = Doc(self.nlp.vocab,words=doc)
            if self.remove_stops is True:     
                doc = [t for t in doc if t.is_stop is False]

           # This functionality does not currently work on spacy end
           # if self.remove_oov is True:
           #     doc = [t for t in doc if t.is_oov is True] # keep if in vocab
            
            if self.keep_alpha is True:
                doc = [t for t in doc if t.is_alpha is True]

            if self.keep_alpha_nums is True:
                doc = [t for t in doc if t.is_alpha or t.is_digit is True]
                        
            if self.remove_punct is True:
                doc = [t for t in doc if t.is_punct is False]
            
            if self.remove_nums is True:
                doc = [t for t in doc if t.like_num is False]
            
            if self.remove_email is True:
                doc = [t for t in doc if t.like_email is False]
            
            if self.remove_url is True:
                doc = [t for t in doc if t.like_url is False]
            
            tkns.append([t.text for t in doc])

        
        return tkns 
    
    def nltk(self, docs):
        '''
        
        :param docs {list[list[str]]} 

        :returns {list[list[str]]} tokenized docs

        '''

        # Tokenize
        tkns = []
        for doc in docs:
        
            if self.remove_stops is True:
                doc = [w for w in doc if w not in self.stop_words]
            
            if self.remove_oov is True:
                doc = [w for w in doc if w in self.vocab]

            tkns.append(doc)
            
        return tkns        

    def frequency(self, docs):
        '''
        Remove words from the documents with frequency count under
        the threshold set by user. Threshold is exclusive (does not
        include the threshold itself).

        :params docs {list[list[str]]} documents

        :returns {list[list[str]]} filtered documents

        '''
        
        # Get frequency counts from doc
        freq_counts = {}
        for doc in docs:
            for w in doc:
                if w in freq_counts:
                    freq_counts[w] += 1
                else:
                    freq_counts[w] = 1

        # Get list of words that are to be removed
        remove_list = [w for w in freq_counts if freq_counts[w] < self.threshold]
        
        # Make new list of documents without words in remove_list
        return [[w for w in doc if w not in remove_list] for doc in docs]

if __name__ == "__main__":
    # Must pass in filename to load a list of strings
    if len(sys.argv) is not 2:
        print("Usage: python3 TokenFilter.py <docs>")
        print("docs: path to a pickled list of strings")
        sys.exit()
    filename = sys.argv[1]
    with open(ROOT_PATH + '/' + filename, 'rb') as f:
        docs = pkl.load(f)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on 6/25/18 

@author: John Sigmon

Last modified: 6/27/18
"""

import os
import sys
import json
import pickle as pkl
import logging.config

logger = logging.getLogger(__name__)
ROOT_PATH = os.path.dirname(os.path.realpath(__file__))

class Stemmer():
    
    apply = None
    nlp = None
    stemmer = None
    lemmatizer = None

    def __init__(self, name, stemmer=None, lemmatizer=None):
        self.dispatch_fun = {
            'nltk': self.nltk,
            'spacy': self.spacy
            }
        if name: 
            self.apply = self.dispatch_fun[name]
        self.stemmer = stemmer
        self.lemmatizer = lemmatizer
        
        if name is 'spacy':
            import en_core_web_sm
            self.nlp = en_core_web_sm.load()

    def nltk(self, docs):
        '''
        Runs NLTK stemmer or lemmatizer. Options are porter and snowball
        stemmer, or wordnet lemmatizer.

        :params docs {list[list[str]]}
       
        :returns {list[list[str]]} stems of words or tokens

        '''
        # No default stemmer currently        
        if self.stemmer == 'porter':
            from nltk.stem import PorterStemmer
            stemmer = PorterStemmer()
        if self.stemmer == 'snowball':
            from nltk.stem import SnowballStemmer
            stemmer = SnowballStemmer('english')
        if self.lemmatizer == 'wordnet':
            from nltk.stem import WordNetLemmatizer
            lemmatizer = WordNetLemmatizer()
        
        # Logic works if functionality not expanded. Check if expanded
        if self.stemmer is not None:
            stems = [[stemmer.stem(w) for w in doc] for doc in docs]
        else:
            stems = [[lemmatizer.lemmatize(w) for w in doc] for doc in docs]
        
        logger.debug("Type of return: %s", type(stems))
        logger.debug("Length: %d", len(stems))
        logger.debug("Length of first entry: %d", len(stems[0]))
        return stems        
    
    def spacy(self, docs):
        '''
        
        
        :params docs {list[list[str]]}
       
        :returns {list[list[str]]} stems of words or tokens

        '''
  
        # Slower but needed format if more logic is added
        #stems = []
        #for doc in docs:
            # No other options yet, just lemmatize
        #    stems.append([w.lemma_ for w in Doc(self.nlp.vocab, words=doc)])

        #  faster until more functionality needed
        from spacy.tokens import Doc
        return [[w.lemma_ for w in Doc(self.nlp.vocab, words=doc)] for doc in docs]

if __name__ == "__main__":
    import Tokenizer as ct
    # Must pass in filename to load a list of strings
    if len(sys.argv) is not 2:
        print("Usage: python3 Stemmer.py <docs>")
        print("docs: path to a pickled list of strings")
        sys.exit()
    filename = sys.argv[1]
    with open(ROOT_PATH + '/' + filename, 'rb') as f:
        docs = pkl.load(f)
    t = ct.Tokenizer('nltk')
    s = Stemmer('nltk', 'snowball')
    docs = t.apply(docs[:10])
    s.apply(docs)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Jun 21 10:56:01 2018

@author: Sanjana Kapoor 
@author: John Sigmon

Last modified: 6/26/18
"""

import os
import sys
import json
import pickle as pkl
import spacy
import en_core_web_sm
import logging.config
from nltk.tokenize import word_tokenize
from spacy.attrs import ORTH, LEMMA

logger = logging.getLogger()
ROOT_PATH = os.path.dirname(os.path.realpath(__file__))

class Tokenizer():
    
    apply = None
    dispatch_fun = None
    to_lower = True

    def __init__(self, name, *params):
        '''

        :param name {str} name of the tokenizer you wish to use

        :param *params {dict} params should be a python dictionary

        '''
        
        self.dispatch_fun = {
                'spacy' : self.spacy,
                'nltk' : self.nltk,
                }
        if name: 
            self.apply = self.dispatch_fun[name]
        for key in params:
            setattr(self, key, params[key])
        
        # Setup big files for vocab etc
        if name == 'spacy':
            import en_core_web_sm
            self.nlp = en_core_web_sm.load()

    def spacy(self, docs):
        '''
        Spacy tokenizer supports adding special cases. 

        The method takes the input docs, tokenizes them, and returns a list
        of lists of strings (tokens).

        : params {list[str]} documents to be tokenized

        : returns {list[list[str]]} tokenized documents

        '''
        
        # Tokenize
        tkns = []
        for doc in docs:
            if self.to_lower is True:
                doc = doc.lower()
            doc = self.nlp.tokenizer(doc)
            tkns.append([t.text for t in doc])

        return tkns 

    def nltk(self, docs):
        '''
        
        :param docs {list[str]} 

        :returns {list[list[str]]} tokenized docs

        '''
        
        # Tokenize
        tkns = []
        for doc in docs:
            if self.to_lower is True:
                doc = doc.lower()
            doc = word_tokenize(doc)
            tkns.append(doc)
        
        return tkns        
    
if __name__ == "__main__":
    # Must pass in filename to load a list of strings
    if len(sys.argv) is not 2:
        print("Usage: python3 Tokenizer.py <docs>")
        print("docs: path to a pickled list of strings")
        sys.exit()
    filename = sys.argv[1]
    with open(ROOT_PATH + '/' + filename, 'rb') as f:
        docs = pkl.load(f)
    
    # First make sure it works without specifying params
    #t_1 = Tokenizer('spacy')
    #t_1.apply(docs[:10])

    #t_2 = Tokenizer('nltk')
    #t_2.apply(docs[:10])

    # Then try arbitrary unsupported params
    #t_3 = Tokenizer('spacy', {'unsupportedparam': 'nonsense'})
    #t_3.apply(docs[:10])
    
    # Then try add_special_case

    test_case = [("don't", [{ORTH: "do"}, {ORTH: "n't", LEMMA: "not"}])]
    params = {'add_special_case': test_case, 'lemmatize': True, 'remove_stops': True}
    tester = Tokenizer('spacy', params)
    tester.apply(docs[:20])




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Jun 21 10:02:44 2018

@author: s0k00rp
@author: John Sigmon

Last modified: 6/22/18
"""

import pickle as pkl
import sys
import os
import json
import logging.config

logger = logging.getLogger()
ROOT_PATH = os.path.dirname(os.path.realpath(__file__))

class Pipeline:
    
    steps = None

    def __init__(self, *args):
        """
        Set up basic pipeline attributes
        """
        
        self.steps = args

    def apply(self, docs):
        for step in self.steps:
            logger.info("Running step {}".format(step))
            docs = step.apply(docs)
        return docs

if __name__=="__main__":
    import Tokenizer as ct
    # Must pass in filename to load a list of strings
    if len(sys.argv) is not 2:
        print("Usage: python3 Pipeline.py <docs>")
        print("docs: path to a pickled list of strings")
        sys.exit()
    filename = sys.argv[1]
    with open(ROOT_PATH + '/' + filename, 'rb') as f:
        docs = pkl.load(f)

    t = ct.Tokenizer('spacy')
    pipeline = Pipeline(t)
    pipeline.apply(docs[:10])



#! /usr/bin/env python3

# Author John Sigmon
# Created 6/28/18
# Last modified 6/28/18

from setuptools import setup
import os
import sys

_ = os.path.abspath(os.path.dirname(__file__))

#try:
#    from pypandoc import convert
#    read_md = lambda f: convert(f, 'rst')
#except ImportError:
#    print("warning: pypandoc module not found, could not convert Markdown to RST")
#    read_md = lambda f: open(f, 'r').read()

#with open('README.md', 'rb') as f:
#    readme = f.read()

requirements = [
    'spacy',
    'nltk',
    ]

setup(
    name='text_pipeline',
    version='1.2.1',
    description='A module for processing text documents in preparation for' +
        ' various NLP models or other tasks that required cleaned text.',
    #long_description=readme,
    #long_description_content_type='text/markdown',
    author='John Sigmon and Sanjana Kapoor',
    author_email='johnsigmon@gmail.com, sanjanakapoor793@gmail.com',
    packages=['text_pipeline'],
    license='MIT',
    install_requires=requirements
)
